@@ -35,14 +35,13 @@
 #import "VLCActivityManager.h"
 #import "VLCDropboxConstants.h"
 #import <ObjectiveDropboxOfficial/ObjectiveDropboxOfficial.h>
+#import "VLCPlaybackNavigationController.h"
+#import "PAPasscodeViewController.h"
 
 NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorized";
 
-#define BETA_DISTRIBUTION 1
-
 @interface VLCAppDelegate () <VLCMediaFileDiscovererDelegate>
 {
-    BOOL _passcodeValidated;
     BOOL _isRunningMigration;
     BOOL _isComingFromHandoff;
     VLCWatchCommunication *_watchCommunication;
@@ -100,20 +99,11 @@ NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorize
     [hockeyManager configureWithBetaIdentifier:@"0114ca8e265244ce588d2ebd035c3577"
                                 liveIdentifier:@"c95f4227dff96c61f8b3a46a25edc584"
                                       delegate:nil];
+    [hockeyManager startManager];
 
     // Configure Dropbox
     [DBClientsManager setupWithAppKey:kVLCDropboxAppKey];
 
-    // Configure the SDK in here only!
-    [hockeyManager startManager];
-    [hockeyManager.authenticator authenticateInstallation];
-
-    /* listen to validation notification */
-    [[NSNotificationCenter defaultCenter] addObserver:self
-                                             selector:@selector(passcodeWasValidated:)
-                                                 name:VLCPasscodeValidated
-                                               object:nil];
-
     // Change the keyboard for UISearchBar
     [[UITextField appearance] setKeyboardAppearance:UIKeyboardAppearanceDark];
     // For the cursor
@@ -131,18 +121,23 @@ NSString *const VLCDropboxSessionWasAuthorized = @"VLCDropboxSessionWasAuthorize
     self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
     // enable crash preventer
     void (^setupBlock)() = ^{
-        _libraryViewController = [[VLCLibraryViewController alloc] init];
-        VLCSidebarController *sidebarVC = [VLCSidebarController sharedInstance];
-        VLCNavigationController *navCon = [[VLCNavigationController alloc] initWithRootViewController:_libraryViewController];
-        sidebarVC.contentViewController = navCon;
+        __weak typeof(self) weakSelf = self;
+        void (^setupLibraryBlock)() = ^{
+            _libraryViewController = [[VLCLibraryViewController alloc] init];
+            VLCNavigationController *navCon = [[VLCNavigationController alloc] initWithRootViewController:_libraryViewController];
 
-        VLCPlayerDisplayController *playerDisplayController = [VLCPlayerDisplayController sharedInstance];
-        playerDisplayController.childViewController = sidebarVC.fullViewController;
+            VLCSidebarController *sidebarVC = [VLCSidebarController sharedInstance];
+            sidebarVC.contentViewController = navCon;
 
-        self.window.rootViewController = playerDisplayController;
+            VLCPlayerDisplayController *playerDisplayController = [VLCPlayerDisplayController sharedInstance];
+            playerDisplayController.childViewController = sidebarVC.fullViewController;
+
+            weakSelf.window.rootViewController = playerDisplayController;
+        };
+        VLCNavigationController *navCon = [[VLCNavigationController alloc] initWithRootViewController:[[UIViewController alloc] init]];
+        self.window.rootViewController = navCon;
         [self.window makeKeyAndVisible];
-
-        [self validatePasscode];
+        [self validatePasscodeIfNeededWithCompletion:setupLibraryBlock];
 
         BOOL spotlightEnabled = ![[VLCKeychainCoordinator defaultCoordinator] passcodeLockEnabled];
         [[MLMediaLibrary sharedMediaLibrary] setSpotlightIndexingEnabled:spotlightEnabled];
@@ -405,9 +400,20 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
 
 - (void)applicationWillResignActive:(UIApplication *)application
 {
-    _passcodeValidated = NO;
-    [self.libraryViewController setEditing:NO animated:NO];
-    [self validatePasscode];
+    //Touch ID is shown 
+    if ([_window.rootViewController.presentedViewController isKindOfClass:[UINavigationController class]]){
+        UINavigationController *navCon = (UINavigationController *)_window.rootViewController.presentedViewController;
+        if ([navCon.topViewController isKindOfClass:[PAPasscodeViewController class]]){
+            return;
+        }
+    }
+    __weak typeof(self) weakself = self;
+    [self validatePasscodeIfNeededWithCompletion:^{
+        [weakself.libraryViewController updateViewContents];
+        if ([VLCPlaybackController sharedInstance].isPlaying){
+            [[VLCPlayerDisplayController sharedInstance] pushPlaybackView];
+        }
+    }];
     [[MLMediaLibrary sharedMediaLibrary] applicationWillExit];
 }
 
@@ -423,7 +429,6 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
 
 - (void)applicationWillTerminate:(UIApplication *)application
 {
-    _passcodeValidated = NO;
     [[NSUserDefaults standardUserDefaults] synchronize];
 }
 
@@ -464,30 +469,15 @@ didFailToContinueUserActivityWithType:(NSString *)userActivityType
 
 #pragma mark - pass code validation
 
-- (void)passcodeWasValidated:(NSNotification *)aNotifcation
-{
-    _passcodeValidated = YES;
-    [self.libraryViewController updateViewContents];
-    if ([VLCPlaybackController sharedInstance].isPlaying)
-        [[VLCPlayerDisplayController sharedInstance] pushPlaybackView];
-}
-
-- (BOOL)passcodeValidated
-{
-    return _passcodeValidated;
-}
-
-- (void)validatePasscode
+- (void)validatePasscodeIfNeededWithCompletion:(void(^)(void))completion
 {
     VLCKeychainCoordinator *keychainCoordinator = [VLCKeychainCoordinator defaultCoordinator];
 
-    if (!_passcodeValidated && [keychainCoordinator passcodeLockEnabled]) {
+    if ([keychainCoordinator passcodeLockEnabled]) {
         [[VLCPlayerDisplayController sharedInstance] dismissPlaybackView];
-
-        [keychainCoordinator validatePasscode];
+        [keychainCoordinator validatePasscodeWithCompletion:completion];
     } else {
-        _passcodeValidated = YES;
-        [self passcodeValidated];
+        completion();
     }
 }
 
